package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.visitor.AttributModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturnException;
import de.fhdw.wtf.common.token.Token;

/**
 * A concrete {@link AttributeModifier}. An {@link Attribute} with this modifier will be generated as <code>final</code>
 * .
 */
public final class AttributeModifierPrior extends AttributeModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 2261377073630110594L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 39023;
	
	/**
	 * Private Constructor of {@link AttributeModifierPrior}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private AttributeModifierPrior(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
		
	}
	
	/**
	 * Creates a {@link AttributeModifierPrior}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return The AttributeModifierPrior-Object.
	 */
	public static AttributeModifierPrior create(final Token firstToken, final Token lastToken) {
		return new AttributeModifierPrior(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link AttributeModifierPrior}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @return The AttributeModifierPrior-Object.
	 */
	public static AttributeModifierPrior create(final Token firstToken) {
		return new AttributeModifierPrior(firstToken, null);
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof AttributeModifierPrior;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	@Override
	public String toString() {
		return de.fhdw.wtf.common.constants.parser.AstDescriptionConstants.MODIFIER_PRIOR;
	}
	
	@Override
	public boolean accept(final AttributModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final AttributeModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final AttributeModifierVisitorReturnException<X, Y> visitor) throws Y {
		return visitor.handle(this);
	}
	
}
