package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.visitor.AttributModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturnException;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Token;

/**
 * A concrete {@link AttributeModifier}. An {@link Attribute} with this modifier will be generated with inverse-Getter.
 */
public final class AttributeModifierSymmetric extends AttributeModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -5971453819738161098L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 2932098;
	
	private final IdentifierToken identifierToken;
	
	public IdentifierToken getIdentifierToken() {
		return this.identifierToken;
	}
	
	/**
	 * Private Constructor of {@link AttributeModifierSymmetric}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param identifierToken
	 *            identifierToken
	 * @param lastToken
	 *            lastToken
	 */
	private AttributeModifierSymmetric(final Token firstToken,
			final IdentifierToken identifierToken,
			final Token lastToken) {
		super(firstToken, lastToken);
		this.identifierToken = identifierToken;
	}
	
	/**
	 * Creates a {@link AttributeModifierSymmetric}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param identifierToken
	 *            identifierToken
	 * @param lastToken
	 *            lastToken
	 * @return The AttributeModifierSymmetric-Object
	 */
	public static AttributeModifierSymmetric create(final Token firstToken,
			final IdentifierToken identifierToken,
			final Token lastToken) {
		return new AttributeModifierSymmetric(firstToken, identifierToken, lastToken);
	}
	
	@Override
	public boolean accept(final AttributModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final AttributeModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final AttributeModifierVisitorReturnException<X, Y> visitor) throws Y {
		return visitor.handle(this);
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof AttributeModifierSymmetric;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
}
