package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitorReturn;
import de.fhdw.wtf.common.token.Token;

/**
 * A concrete {@link ClassModifier}. A {@link ClassType} with this modifier will be generated as an abstract class.
 */
public final class ClassModifierAbstract extends ClassModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 6246401567888957022L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 23092389;
	
	/**
	 * Constructor of {@link ClassModifierAbstract}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private ClassModifierAbstract(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link ClassModifierAbstract}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return The {@link ClassModifierAbstract}-Object
	 */
	public static ClassModifierAbstract create(final Token firstToken, final Token lastToken) {
		return new ClassModifierAbstract(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link ClassModifierAbstract}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @return The {@link ClassModifierAbstract}-Object
	 */
	public static ClassModifierAbstract create(final Token firstToken) {
		return new ClassModifierAbstract(firstToken, null);
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof ClassModifierAbstract;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	@Override
	public String toString() {
		return de.fhdw.wtf.common.constants.parser.AstDescriptionConstants.MODIFIER_ABSTRACT;
	}
	
	@Override
	public boolean accept(final ClassModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final ClassModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
}
