package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitor;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorException;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorReturnException;
import de.fhdw.wtf.common.ast.visitor.TypeVisitor;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorException;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorReturnException;
import de.fhdw.wtf.common.token.Token;

/**
 * {@link Type} which contains other {@link Type}s.
 */
public abstract class CompositeType extends RealType {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -846566959970409182L;
	
	/**
	 * Constructor of {@link CompositeType}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	protected CompositeType(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	@Override
	public void accept(final TypeVisitor v) {
		v.handle(this);
	}
	
	@Override
	public <X> X accept(final TypeVisitorReturn<X> v) {
		return v.handle(this);
	}
	
	@Override
	public <X extends Exception> void accept(final TypeVisitorException<X> v) throws X {
		v.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final TypeVisitorReturnException<X, Y> v) throws Y {
		return v.handle(this);
	}
	
	/**
	 * Accept a Visitor and calls handle of it.
	 * 
	 * @param v
	 *            Visitor
	 */
	public abstract void accept(CompositeTypeVisitor v);
	
	/**
	 * Accept a Visitor and calls handle of it.
	 * 
	 * @param <X>
	 *            ReturnType
	 * @param v
	 *            Visitor
	 * @return X
	 */
	public abstract <X> X accept(CompositeTypeVisitorReturn<X> v);
	
	/**
	 * Accept a Visitor and calls handle of it.
	 * 
	 * @param v
	 *            Visitor
	 * @param <X>
	 *            ExceptionType
	 * @throws X
	 *             Exception
	 */
	public abstract <X extends Exception> void accept(CompositeTypeVisitorException<X> v) throws X;
	
	/**
	 * Accept a Visitor and calls handle of it.
	 * 
	 * @param <X>
	 *            ReturnType
	 * @param v
	 *            Visitor
	 * @param <Y>
	 *            ExceptionType
	 * @return X
	 * @throws Y
	 *             Exception
	 */
	public abstract <X, Y extends Exception> X accept(CompositeTypeVisitorReturnException<X, Y> v) throws Y;
}
