package de.fhdw.wtf.common.exception.referencer;

import de.fhdw.wtf.common.exception.walker.TaskException;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * It is a general abstract Exception for Referencer-Exceptions.
 * 
 */
public abstract class ReferencerException extends TaskException {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 6930888853075370916L;
	
	/**
	 * Start-Position.
	 */
	private final Position posStart;
	
	/**
	 * End-Position.
	 */
	private final Position posEnd;
	
	/**
	 * Constructor of {@link ReferencerException}.
	 * 
	 * @param token
	 *            token
	 * @param message
	 *            message
	 */
	public ReferencerException(final Token token, final String message) {
		super(message);
		this.posStart = token.getPosition();
		this.posEnd =
				Position.create(this.posStart.getFilePath(), this.posStart.getLineNumber(), this.posStart.getPosition()
						+ token.getLength(), this.posStart.getPosition() + token.getLength());
	}
	
	/**
	 * Start-Position.
	 * 
	 * @return Start-Position
	 */
	public Position getPosStart() {
		return this.posStart;
	}
	
	/**
	 * End-Position.
	 * 
	 * @return End-Position
	 */
	public Position getPosEnd() {
		return this.posEnd;
	}
}
