package de.fhdw.wtf.common.stream;

import java.util.ArrayList;
import java.util.List;

import de.fhdw.wtf.common.token.Token;

/**
 * Implementation of a tokenstream that uses a list of token.
 * 
 */
public final class SimpleTokenStream implements TokenStream {
	
	/**
	 * Representing all tokens in the correct order.
	 */
	private final List<Token> data;
	
	/**
	 * Constructor of {@link SimpleTokenStream}.
	 */
	private SimpleTokenStream() {
		this.data = new ArrayList<>();
	}
	
	/**
	 * Constructor of {@link SimpleTokenStream}.
	 * 
	 * @param data
	 *            data
	 */
	private SimpleTokenStream(final List<Token> data) {
		this.data = data;
	}
	
	/**
	 * Creates a {@link SimpleTokenStream}-Object.
	 * 
	 * @return The {@link SimpleTokenStream}-Object
	 */
	public static SimpleTokenStream create() {
		return new SimpleTokenStream();
	}
	
	@Override
	public void add(final Token t) {
		this.data.add(t);
	}
	
	@Override
	public Token next() {
		final Token result = this.data.get(0);
		this.data.remove(0);
		return result;
	}
	
	@Override
	public boolean hasNext() {
		return !this.data.isEmpty();
	}
	
	@Override
	public boolean removeFirst() {
		if (this.hasNext()) {
			this.data.remove(0);
			return true;
		}
		return false;
	}
	
	@Override
	public Token peek() {
		return this.data.get(0);
	}
	
	@Override
	public TokenStream copy() {
		final List<Token> copiedData = new ArrayList<>();
		copiedData.addAll(this.data);
		return new SimpleTokenStream(copiedData);
	}
	
}
