package de.fhdw.wtf.common.token;

/**
 * This token contains a not-recognized string. e.g. invalid character sequences in the original input.
 * 
 */
public final class InvalidToken extends MultiCharToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -8125327670667747456L;
	
	/**
	 * Private Constructor for InvalidToken. Use create(...)-Factory instead!
	 * 
	 * @param invalidString
	 *            initial invalid string
	 * @param position
	 *            start position of this token in the original input
	 */
	private InvalidToken(final String invalidString, final Position position) {
		super(invalidString, position);
	}
	
	/**
	 * Factory for a {@link InvalidToken}.
	 * 
	 * @param invalidChar
	 *            initial invalid single-char
	 * @param position
	 *            start position of this token in the original input
	 * @return a new instance of this token
	 */
	public static InvalidToken create(final char invalidChar, final Position position) {
		return new InvalidToken(Character.toString(invalidChar), position);
	}
	
	/**
	 * Factory for a {@link InvalidToken}.
	 * 
	 * @param invalidString
	 *            initial invalid string.
	 * @param position
	 *            start position of this token in the original input
	 * @return a new instance of this token
	 */
	public static InvalidToken create(final String invalidString, final Position position) {
		return new InvalidToken(invalidString, position);
	}
	
	@Override
	public boolean isInvalidToken() {
		return true;
	}
	
	/**
	 * Invalid character-sequence represented by this token.
	 * 
	 * @return the invalid string of this token
	 */
	public String getInvalidString() {
		return this.getTokenString();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		if (t.isInvalidToken()) {
			final InvalidToken i = (InvalidToken) t;
			return i.getInvalidString().equals(this.getInvalidString());
		}
		return false;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return this.getInvalidString().hashCode();
	}
	
	@Override
	public boolean equals(final Object o) {
		if (o instanceof InvalidToken) {
			return super.equals(o);
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}
}
