package de.fhdw.wtf.common.token.keywords;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the symmetric-modifier as a token in WTF.
 */
public final class SymmetricToken extends AttributeModifierToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -1610618887460906823L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 12312355;
	
	/**
	 * Defines constant "symmetric" for {@link SymmetricToken}.
	 */
	private static final String TEXTUAL_REPRESENTATION = "symmetric";
	
	/**
	 * Private constructor for {@link SymmetricToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the {@link AttributeModifierToken} <prior> in the original input.
	 */
	private SymmetricToken(final Position position) {
		super(position);
	}
	
	/**
	 * Factory method for {@link SymmetricToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link AttributeModifierToken} in the original input.
	 * @return a new instance of this {@link AttributeModifierToken}.
	 */
	public static SymmetricToken create(final Position position) {
		return new SymmetricToken(position);
	}
	
	@Override
	public int getLength() {
		return SymmetricToken.TEXTUAL_REPRESENTATION.length();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isSymmetricToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymmetricToken.TEXTUAL_REPRESENTATION;
	}
	
	@Override
	public boolean isSymmetricToken() {
		return true;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
