package de.fhdw.wtf.tooling.editor;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;

import de.fhdw.wtf.tooling.editor.proposals.AttributeModifierProposal;
import de.fhdw.wtf.tooling.editor.proposals.BaseTypeProposal;
import de.fhdw.wtf.tooling.editor.proposals.ClassModifierProposal;
import de.fhdw.wtf.tooling.editor.proposals.ClassProposal;
import de.fhdw.wtf.tooling.editor.proposals.ContextProposal;
import de.fhdw.wtf.tooling.editor.proposals.GroupProposal;
import de.fhdw.wtf.tooling.editor.proposals.OperationBaseTypeProposal;
import de.fhdw.wtf.tooling.editor.proposals.UserTypeProposal;

public class CompletionProcessor implements IContentAssistProcessor {
	
	private final IContextInformation[] NO_CONTEXTS = {};
	private final char[] ACTIVATION_CHARS = { 'c', 'g', 'i', 's', 'p', 't', 'f', 'm', 'o', 'a', 'I', 'S' };
	private final Collection<ContextProposal> proposals;
	
	public CompletionProcessor() {
		this.proposals = new Vector<>();
		this.proposals.add(BaseTypeProposal.create());
		this.proposals.add(GroupProposal.create());
		this.proposals.add(ClassProposal.create());
		this.proposals.add(AttributeModifierProposal.create());
		this.proposals.add(ClassModifierProposal.create());
		this.proposals.add(OperationBaseTypeProposal.create());
		this.proposals.add(UserTypeProposal.create());
	}
	
	@Override
	public ICompletionProposal[] computeCompletionProposals(final ITextViewer viewer, final int offset) {
		final Collection<ICompletionProposal> result = new Vector<>();
		for (final ContextProposal proposal : this.proposals) {
			final ICompletionProposal[] checkedProposals = proposal.getProposals(viewer, offset);
			if (checkedProposals != null) {
				for (final ICompletionProposal current : checkedProposals) {
					if (current == null) {
						continue;
					}
					result.add(current);
				}
			}
		}
		
		final ICompletionProposal[] props = new CompletionProposal[result.size()];
		return result.toArray(props);
	}
	
	@Override
	public IContextInformation[] computeContextInformation(final ITextViewer viewer, final int offset) {
		return this.NO_CONTEXTS;
	}
	
	@Override
	public char[] getCompletionProposalAutoActivationCharacters() {
		return this.ACTIVATION_CHARS;
	}
	
	@Override
	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}
	
	@Override
	public String getErrorMessage() {
		return null;
	}
	
	@Override
	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}
	
}
