package de.fhdw.wtf.tooling.editor;

import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleConstants;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.IConsoleView;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;

/**
 * A class which allows the Editor to write Information on the Console.
 *
 */
public class ConsoleWriter {
	
	private static final String WTF_EDITOR_CONSOLE_NAME = "WTFEditorConsole";
	private static MessageConsole consoleInstance;
	private static MessageConsoleStream messageConsoleStream;
	
	private static MessageConsole getInstance() {
		if (consoleInstance == null) {
			consoleInstance = findConsole(WTF_EDITOR_CONSOLE_NAME);
		}
		return consoleInstance;
	}
	
	private static MessageConsoleStream getMessageConsoleStream() {
		if (messageConsoleStream == null) {
			messageConsoleStream = getInstance().newMessageStream();
		}
		return messageConsoleStream;
	}
	
	private static MessageConsole findConsole(final String name) {
		final ConsolePlugin plugin = ConsolePlugin.getDefault();
		final IConsoleManager consoleManager = plugin.getConsoleManager();
		for (final IConsole console : consoleManager.getConsoles()) {
			if (name.equals(console.getName())) {
				return (MessageConsole) console;
			}
		}
		final MessageConsole console = new MessageConsole(name, null);
		consoleManager.addConsoles(new IConsole[] { console });
		return console;
	}
	
	public static void println(final String message) {
		getMessageConsoleStream().println(message);
	}
	
	public static void print() {
		getMessageConsoleStream().println();
	}
	
	public static void print(final String message) {
		getMessageConsoleStream().print(message);
	}
	
	public static void setColor(final Color newColor) {
		getMessageConsoleStream().setColor(newColor);
	}
	
	public static void clearConsole() {
		getInstance().clearConsole();
	}
	
	public static void displayConsoleView() {
		final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		final IConsole myConsole = getInstance();
		final String id = IConsoleConstants.ID_CONSOLE_VIEW;
		IConsoleView view;
		try {
			view = (IConsoleView) page.showView(id);
			view.display(myConsole);
		} catch (final PartInitException e) {
			e.printStackTrace();
		}
	}
}
