package de.fhdw.wtf.walker.walker;

import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.exception.walker.TaskException;
import de.fhdw.wtf.common.task.DependencyTask;
import de.fhdw.wtf.common.task.TaskExecutor;

/**
 * Walker through all type definitions within the WTF model.
 */
public abstract class WalkerTaskForTypes extends DependencyTask {
	
	/**
	 * Constructor for {@link WalkerTaskForTypes}.
	 * 
	 * @param taskmanager
	 *            Taskmanager
	 */
	public WalkerTaskForTypes(final TaskExecutor taskmanager) {
		super(taskmanager);
	}
	
	/**
	 * This handle will be called with each type in a given model ONCE PER TYPE!
	 * 
	 * @param c
	 *            type to be handled
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void handleType(Type c) throws TaskException;
	
	/**
	 * This operation will be called ONCE at the end of the task!
	 * 
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void finalizeTask() throws TaskException;
	
	/**
	 * This operation will be called ONCE at the begin of the task!
	 * 
	 * @throws TaskException
	 *             may be thrown by the implementing task
	 */
	public abstract void beginTask() throws TaskException;
	
}
